/*
 * Copyright (C) 2004 Oliver Horn
 *
 * You may use and modify this package for any purpose. Redistribution is
 * permitted, in both source and binary form, provided that this notice
 * remains intact in all source distributions of this package.
 */

package com.oliverh.xsltext.tokenizer;

import java.io.*;
import javax.swing.text.Segment;
import javax.xml.transform.TransformerException;
import org.jedit.syntax.*;

/**
 * @author Oliver Horn
 */
public class JEditSyntaxTokenizer extends Tokenizer {
		
	public JEditSyntaxTokenizer() {
		this(null);
	}
	
	public JEditSyntaxTokenizer(TokenMarker marker) {
		setMarker(marker);
	}
	
	public void setMarker(TokenMarker marker) {
		this.marker = marker;
	}

	public TokenMarker getMarker() {
		return marker;
	}

	public void tokenize(Reader in) throws IOException, TransformerException {
		final TokenConsumer consumer = getTokenConsumer();
		final BufferedReader reader = new BufferedReader(in);
		int lineNo = 1;
		String line = reader.readLine();
		while (line != null) {
			final char[] ch = line.toCharArray();
			if (lineNo == 1 && ch.length == 0) {
				line = reader.readLine();
				continue;
			}
			consumer.startLine(lineNo);

			if (getMarker() != null) {
				getMarker().insertLines(getMarker().getLineCount(), 1);			
				final Segment s = new Segment(ch, 0, ch.length);
				int index = 0;
				Token token = marker.markTokens(s, getMarker().getLineCount()-1);
				while (token.id != Token.END) {
					final int type = getTokenType(token);
					if (type != 0) {
						consumer.startToken(type);
					}
					consumer.characters(ch, index, token.length);
					if (type != 0) {
						consumer.endToken(type);
					}
					index += token.length;
					token = token.next;
				}
			} else {
				consumer.characters(ch, 0, ch.length);
			}
			consumer.endLine(lineNo);
			line = reader.readLine();
			lineNo++;
		}		
	}
	
	protected int getTokenType(Token token) {
		switch (token.id) {
		case Token.KEYWORD1:
		case Token.KEYWORD2:
		case Token.KEYWORD3:
			return TokenConsumer.KEYWORD;
		case Token.LITERAL1:
		case Token.LITERAL2:
			return TokenConsumer.LITERAL;
		case Token.COMMENT1:
		case Token.COMMENT2:
			return TokenConsumer.COMMENT;
		case Token.OPERATOR:
			return TokenConsumer.OPERATOR;
		default:
			return 0;
		}		
	}
	
	private TokenMarker marker;
}
