/*
 * Copyright (C) 2004 Oliver Horn
 *
 * You may use and modify this package for any purpose. Redistribution is
 * permitted, in both source and binary form, provided that this notice
 * remains intact in all source distributions of this package.
 */

package com.oliverh.xsltext.tokenizer;

import java.io.*;
import javax.xml.transform.TransformerException;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.AttributesImpl;
import com.icl.saxon.Context;
import com.icl.saxon.om.*;

/**
 * @author Oliver Horn
 */
public class SaxonTokenizer {

	private static String[] ROLES =
			{ null, "keyword", "literal", "operator", "comment" };

	private final static Attributes NO_ATTS = new AttributesImpl();
	
	private final static char[] NEWLINE = {'\n'};

	public static boolean supportsSyntax(String language) throws Exception {
		return Tokenizer.getTokenizer(language) != null;
	}
	
	public static NodeInfo tokenize(Context context, String text, String language) throws Exception {
		try {
			final Builder builder = context.getController().makeBuilder();
			builder.startDocument();

			final Tokenizer tokenizer = Tokenizer.getTokenizer(language);
			if (tokenizer != null) {			
				final int lineName = builder.getNamePool().allocate("", "", "line");
				final int tokenName = builder.getNamePool().allocate("", "", "token");
				tokenizer.setTokenConsumer(new TokenConsumer() {
					public void startLine(int row) throws TransformerException {
						builder.startElement(lineName, NO_ATTS, new int[0], 0);
					}
					public void endLine(int row) throws TransformerException {
						builder.endElement(lineName);
						// builder.characters(NEWLINE, 0, 1);
					}
					public void startToken(int type) throws TransformerException {
						final AttributesImpl atts = new AttributesImpl();
						atts.addAttribute("", "class", "class", "", ROLES[type]);
						builder.startElement(tokenName, atts, new int[0], 0);
					}
					public void endToken(int type) throws TransformerException {
						builder.endElement(tokenName);
					}
					public void characters(char[] buf, int index, int len) throws TransformerException {
						builder.characters(buf, index, len);
					}
				});
				tokenizer.tokenize(new StringReader(text));
			}
			
			builder.endDocument();
			return builder.getCurrentDocument();
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}
}
